/***********************************************************************
 *                                                                     *
 * This file contains the implementation of uart1.h for the            *
 * dsPIC30F3011 device.                                                *
 *                                                                     *
 ***********************************************************************
 *                                                                     * 
 *    Author:         Tom Pycke                                        *
 *    Filename:       uart1.c                                          *
 *    Date:           13/10/2007                                       *
 *    File Version:   1.00                                             *
 *    Other Files Required: uart1.h                                    *
 *                                                                     *
 ***********************************************************************/

#include "micro_frc.h"
#include "uart1.h"


#include <uart.h>

void uart1_open(long baud)
{
	// configure U2MODE
	U1MODEbits.UARTEN = 0;	// Bit15 TX, RX DISABLED, ENABLE at end of func
	U1MODEbits.USIDL = 0;	// Bit13 Continue in Idle
	U1MODEbits.IREN = 0;	// Bit12 No IR translation
	U1MODEbits.RTSMD = 1;	// Bit11 Simplex Mode
	U1MODEbits.UEN = 0;		// Bits8,9 TX,RX enabled, CTS,RTS not
	U1MODEbits.WAKE = 0;	// Bit7 No Wake up (since we don't sleep here)
	U1MODEbits.LPBACK = 0;	// Bit6 No Loop Back
	U1MODEbits.ABAUD = 0;	// Bit5 No Autobaud (would require sending '55')
	U1MODEbits.URXINV = 0;	// Bit4 IdleState = 1  (for dsPIC)
	U1MODEbits.BRGH = 0;	// Bit3 16 clocks per bit period
	U1MODEbits.PDSEL = 0;	// Bits1,2 8bit, No Parity
	U1MODEbits.STSEL = 0;	// Bit0 One Stop Bit
	
	// Load a value into Baud Rate Generator.  Example is for 9600.
	// See section 19.3.1 of datasheet.
	//  U2BRG = (Fcy/(16*BaudRate))-1
	//  U2BRG = (40MHz/(16*38400))-1
	//  U2BRG = 65
	
	U1BRG = (int)(FCY / (16*baud) - 1);	
	
	// Load all values in for U1STA SFR
	U1STAbits.UTXISEL1 = 0;	//Bit15 Int when Char is transferred (1/2 config!)
	
	U1STAbits.UTXINV = 0;	//Bit14 N/A, IRDA config
	U1STAbits.UTXISEL0 = 0;	//Bit13 Other half of Bit15
	U1STAbits.UTXBRK = 0;	//Bit11 Disabled
	U1STAbits.UTXEN = 1;	//Bit10 TX pins controlled by periph
	U1STAbits.URXISEL = 0;	//Bits6,7 Int. on character recieved
	U1STAbits.ADDEN = 0;	//Bit5 Address Detect Disabled

	IPC7 = 0x4400;	// Mid Range Interrupt Priority level, no urgent reason

//	IFS1bits.U1TXIF = 0;	// Clear the Transmit Interrupt Flag
	//IEC1bits.U1TXIE = 1;	// Enable Transmit Interrupts
//  IFS1bits.U1RXIF = 0;	// Clear the Recieve Interrupt Flag
	//IEC1bits.U1RXIE = 1;	// Enable Recieve Interrupts

	U1MODEbits.UARTEN = 1;	// And turn the peripheral on

	U1STAbits.UTXEN = 1;
	// I think I have the thing working now.
}


void uart1_puts(char string[])
{
	putsUART1 ((unsigned int *)string); //library function
	while (BusyUART1());
}



void PrintInteger(unsigned int x, char *buffer)
{
	static char emptyChar = ' ', tmpChar;
	unsigned int radix,
	    i;
	emptyChar = ' ';
	
	// auto-align on 16bit size (655.. = 5 positions)
	for (radix=10000, i=0; i < 5; radix/=10, i++)
	{
		if (x > radix-1) {
			tmpChar =  '0' + (x/radix);
			buffer[i] = tmpChar;
			x = x % radix;
			emptyChar = '0';
		} else
			buffer[i] = emptyChar;	
	}
	if (emptyChar == ' ')
		buffer[4] = '0';
	buffer[5] = '\0';
}


void PrintSignedInteger(int x, void (*printer)(char[]))
{
	static char tmpChar[2] = "0";
	unsigned int radix,
                 i, converted;
	char emptyChar = ' ';
	
	if (x < 0)
	{
		printer("-");
		x*=-1;
	}
	else if (x == 0)
	{
		printer("0");
		return;	
	}
	converted = (unsigned int) x;
	// auto-align on 16bit size (655.. = 5 positions)
	for (radix=10000, i=0; i < 5; radix/=10, i++)
	{
		if (converted > radix-1) {
			tmpChar[0] =  '0' + (converted/radix);
			printer(tmpChar);
			converted = converted % radix;
			emptyChar = '0';
		} else if (emptyChar == '0') {
			printer("0");
		}
	}
}

void rs232_send(int outdata) 
{

union  {
  		int out16;
  		struct {
    			char lbyte;
    			char mbyte;
    			} out8;
		} od;

#define msbyte od.out8.mbyte
#define lsbyte od.out8.lbyte
	
od.out16=outdata;  // copy input to structure

	myputc(msbyte); 
	myputc(lsbyte); 
}

void myputc(char abyte)
{
while(U1STAbits.UTXBF); /* wait if the buffer is full */
U1TXREG = abyte;
}


